Package: numbat
Title: Haplotype-Aware CNV Analysis from scRNA-Seq
URL: https://github.com/kharchenkolab/numbat/,
        https://kharchenkolab.github.io/numbat/
Version: 1.5.2
Authors@R: c(person("Teng","Gao", email="tgaoteng@gmail.com", role=c("cre", "aut")), person("Ruslan", "Soldatov", email="soldatr@mskcc.org", role="aut"), person("Hirak", "Sarkar", email="hirak_sarkar@hms.harvard.edu", role="aut"), person("Evan", "Biederstedt", email="evan.biederstedt@gmail.com", role="aut"), person("Peter", "Kharchenko", email = "peter_kharchenko@hms.harvard.edu", role = "aut"))
Description: A computational method that infers copy number variations (CNVs) in cancer scRNA-seq data and reconstructs the tumor phylogeny. 'numbat' integrates signals from gene expression, allelic ratio, and population haplotype structures to accurately infer allele-specific CNVs in single cells and reconstruct their lineage relationship. 'numbat' can be used to: 1. detect allele-specific copy number variations from single-cells; 2. differentiate tumor versus normal cells in the tumor microenvironment; 3. infer the clonal architecture and evolutionary history of profiled tumors. 'numbat' does not require tumor/normal-paired DNA or genotype data, but operates solely on the donor scRNA-data data (for example, 10x Cell Ranger output). Additional examples and documentations are available at <https://kharchenkolab.github.io/numbat/>. For details on the method please see Gao et al. Nature Biotechnology (2022) <doi:10.1038/s41587-022-01468-y>.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.1.0), Matrix
biocViews:
Imports: ape, caTools, data.table, dendextend, dplyr (>= 1.1.1),
        GenomicRanges, ggplot2, ggraph, ggtree, glue, hahmmr, igraph,
        IRanges, logger, magrittr, methods, optparse, parallel,
        parallelDist, patchwork, purrr, Rcpp, RhpcBLASctl, R.utils,
        scales, scistreer (>= 1.1.0), stats4, stringr, tibble,
        tidygraph, tidyr (>= 1.3.0), vcfR, zoo
Suggests: ggrastr, ggrepel, knitr, matrixStats, testthat (>= 3.0.0),
Config/testthat/edition: 3
LinkingTo: Rcpp, RcppArmadillo, roptim
NeedsCompilation: yes
SystemRequirements: GNU make
Author: Teng Gao [cre, aut],
  Ruslan Soldatov [aut],
  Hirak Sarkar [aut],
  Evan Biederstedt [aut],
  Peter Kharchenko [aut]
Maintainer: Teng Gao <tgaoteng@gmail.com>
RoxygenNote: 7.3.2
Packaged: 2026-02-04 04:56:27 UTC; tenggao
Repository: CRAN
Date/Publication: 2026-02-04 08:20:15 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-30 14:42:58 UTC; unix
Archs: numbat.so.dSYM
