# Generated by roxygen2: do not edit by hand

export(Numbat)
export(aggregate_counts)
export(analyze_bulk)
export(annotate_genes)
export(cnv_heatmap)
export(detect_clonal_loh)
export(get_bulk)
export(get_gtree)
export(plot_bulks)
export(plot_consensus)
export(plot_exp_roll)
export(plot_mut_history)
export(plot_phylo_heatmap)
export(plot_psbulk)
export(plot_sc_tree)
export(run_numbat)
import(Matrix)
import(Rcpp)
import(dplyr)
import(ggplot2)
import(ggraph)
import(glue)
import(logger)
import(patchwork)
import(stringr)
import(tibble)
import(tidygraph)
importFrom(data.table,as.data.table)
importFrom(data.table,fread)
importFrom(data.table,fwrite)
importFrom(ggtree,"%<+%")
importFrom(grDevices,colorRampPalette)
importFrom(hahmmr,dbbinom)
importFrom(hahmmr,dpoilog)
importFrom(hahmmr,fit_lnpois_cpp)
importFrom(hahmmr,forward_back_allele)
importFrom(hahmmr,l_bbinom)
importFrom(hahmmr,l_lnpois)
importFrom(hahmmr,likelihood_allele)
importFrom(hahmmr,logSumExp)
importFrom(hahmmr,run_allele_hmm_s5)
importFrom(hahmmr,run_joint_hmm_s15)
importFrom(igraph,"E<-")
importFrom(igraph,"V<-")
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,ecount)
importFrom(igraph,vcount)
importFrom(methods,as)
importFrom(methods,is)
importFrom(parallel,mclapply)
importFrom(scistreer,annotate_tree)
importFrom(scistreer,get_mut_graph)
importFrom(scistreer,ladderize)
importFrom(scistreer,mut_to_tree)
importFrom(scistreer,perform_nni)
importFrom(scistreer,score_tree)
importFrom(scistreer,to_phylo)
importFrom(stats,as.dendrogram)
importFrom(stats,as.dist)
importFrom(stats,as.ts)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,cutree)
importFrom(stats,dbinom)
importFrom(stats,dnbinom)
importFrom(stats,dnorm)
importFrom(stats,dpois)
importFrom(stats,end)
importFrom(stats,hclust)
importFrom(stats,integrate)
importFrom(stats,is.leaf)
importFrom(stats,model.matrix)
importFrom(stats,na.contiguous)
importFrom(stats,na.omit)
importFrom(stats,optim)
importFrom(stats,p.adjust)
importFrom(stats,pnorm)
importFrom(stats,reorder)
importFrom(stats,rnorm)
importFrom(stats,setNames)
importFrom(stats,start)
importFrom(stats,t.test)
importFrom(utils,combn)
useDynLib(numbat, .registration=TRUE)
